{% load i18n %}
<div id="iommi_{{ query.iommi_dunder_path }}">
    <form{{ query.form.attrs }}>

        {% if query.advanced %}
            {{ query.advanced }}
        {% endif %}

        {{ query.form_container.iommi_open_tag }}
            {{ query.form.render_fields }}
        {{ query.form_container.iommi_close_tag }}

        {{ query.form.errors }}

        <div class="iommi_query_error">
            {% if query.query_error %}
                {{ query.query_error }}
            {% endif %}
        </div>

        {{ query.form.render_actions }}
   </form>
</div>
